﻿/**
*@FILENAME		: KanbanStaion.cs
*@DESCRIPTION	: Kanban application that simulates a kanban process.
*@VERSION		: April 20, 2017
*@PROGRAMMER	: Brian Hinds and Laura Fadrique
*/

using System;
using System.Data;
using System.Data.OleDb;
using System.Windows.Forms;

namespace KanbanStationApplication
{
    public partial class KanbanStation : Form
    {
        /// <summary>
        /// Variables for the Station Kanban Station Application
        /// </summary>
        private string StationConnectionString = "";

        private int ProfileTimer = 0;
        private bool TimerScale = true;
        private int ProfileID = 0;
        private int ProductID = 1;
        private int StationID = 0;
        private const int TimerToSeconds = 100;
        private const int TimerToMinutes = 1000;
        private string buffer = "";
        private int ConfigTimer = 0;
        private bool isRunning = false;
        private const int TimerPercentage = 100;
        private int StationTimerInterval = 0;
        private int count = 0;
        private double avgDefect = 0.00;
        private bool ProfileLoaded = false;

        /// <summary>
        /// Instantiating the classes needed for the kanban station
        /// </summary>
        private StationConnection KanbanConnection = new StationConnection();

        private StationCommands KanbanCommands = new StationCommands();

        public KanbanStation()
        {
            InitializeComponent();
            startBtn.Enabled = false;
            stopBtn.Enabled = false;
            ProfileBox.Enabled = false;
            ProductBox.Enabled = false;
        }

        /// <summary>
        /// Default connection method that execute command strings
        /// </summary>
        /// <param name="CommandString"></param>
        /// <returns></returns>
        private DataTable KanbanCommand(string CommandString)
        {
            OleDbConnection con = new OleDbConnection(StationConnectionString);
            con.Open();
            OleDbCommand cmd = con.CreateCommand();
            cmd.CommandType = CommandType.Text;
            cmd.CommandText = CommandString;
            cmd.ExecuteNonQuery();
            DataTable dt = new DataTable();
            OleDbDataAdapter da = new OleDbDataAdapter(cmd);
            da.Fill(dt);
            con.Close();
            return dt;
        }

        /// <summary>
        /// This method loads the station profiles
        /// </summary>
        private void loadProfile()
        {
            try
            {
                connectionPIC.Image = Properties.Resources.waiting;
                ProfileBox.DataSource = KanbanCommand(KanbanCommands.LoadProfile());
                ProfileBox.DisplayMember = "profile_desc";
                ProfileBox.ValueMember = "profile_id";
                connectionPIC.Image = Properties.Resources.ok;

                ProfileBox.SelectedIndex = 0;
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.ToString());
                connectionPIC.Image = Properties.Resources.error;
            }
        }

        /// <summary>
        /// This method loads the station products
        /// </summary>
        private void loadproduct()
        {
            try
            {
                connectionPIC.Image = Properties.Resources.waiting;
                ProductBox.DataSource = KanbanCommand(KanbanCommands.LoadProduct());
                ProductBox.DisplayMember = "product_desc";
                ProductBox.ValueMember = "product_id";
                connectionPIC.Image = Properties.Resources.ok;
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.ToString());
                connectionPIC.Image = Properties.Resources.error;
            }
        }

        /// <summary>
        /// This method calculate the average time for station operator
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ProfileBox_SelectedIndexChanged(object sender, EventArgs e)
        {

            if (ProfileLoaded)
            {
                LoadProfileConfig();
            }
        }



        /// <summary>
        /// This method starts the station
        /// </summary>
        private void LoadProfileConfig()
        {
            //Get Profile Id from combo
            ProfileID = Convert.ToInt32(ProfileBox.SelectedValue.ToString());

            /// Getting profile average run time 
            GetProfileTimer();

            try
            {
                OleDbConnection con = new OleDbConnection(StationConnectionString);
                con.Open();
                OleDbCommand cmd = new OleDbCommand(KanbanCommands.LoadConfiguration(), con);
                OleDbDataReader ConfigValue = null;
                ConfigValue = cmd.ExecuteReader();
                while (ConfigValue.Read())
                {
                    ConfigTimer = Convert.ToInt32(ConfigValue["avg_assembly"]);
                    TimerScale = Convert.ToBoolean(ConfigValue["scale_flag"]);
                }
                con.Close();
                startBtn.Enabled = true;
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.ToString());
            }
        }


        /// <summary>
        /// This method starts the station
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void startBtn_Click(object sender, EventArgs e)
        {
            StartStation();

            if (isRunning)
            {
                /// Calculating Timer configuration ///
                double factor = 0.00;
                factor = (ProfileTimer + 0.0) / TimerPercentage;
                StationTimerInterval = Convert.ToInt32(factor * ConfigTimer);

                sNumber.Text = StationID.ToString();

                /// Setting timer configuration ///
                if (factor != 1) StationTimerInterval += ConfigTimer;

                /// Not scale to real time ///
                if (TimerScale) { StationTimerInterval *= TimerToSeconds; }

                /// Scale to real Time ///
                if (!TimerScale) { StationTimerInterval *= TimerToMinutes; }

                /// Assigning timer interval ///
                KanbanTimer.Interval = StationTimerInterval;

                /// Start timer ///
                KanbanTimer.Start();

                startBtn.Enabled = false;

                stopBtn.Enabled = true;
            }
        }

        /// <summary>
        /// This method stops the station
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void stopBtn_Click(object sender, EventArgs e)
        {
            KanbanTimer.Stop();
            startBtn.Enabled = true;
            stopBtn.Enabled = false;
            count = 0;
            lblQty.Text = count.ToString();
        }

        /// <summary>
        /// This method assembles and check the bin per define interval
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void KanbanTimer_Tick(object sender, EventArgs e)
        {
            AssembleLamp();
            CheckBin();
        }

        /// <summary>
        /// This method assembles a lamp
        /// </summary>
        private void AssembleLamp()
        {
            try
            {
                int Defect = GetDefect();

                assemblePIC.Image = Properties.Resources.waiting;
                OleDbConnection con = new OleDbConnection(StationConnectionString);
                con.Open();
                OleDbCommand cmd = new OleDbCommand(KanbanCommands.AssembleLamp(), con);
                cmd.CommandType = CommandType.StoredProcedure;
                cmd.Parameters.Add("@paramStationId ", OleDbType.Integer).Value = StationID;
                cmd.Parameters.Add("@paramDateProduction ", OleDbType.Date).Value = DateTime.Now.ToString("yyyy-MM-dd");
                cmd.Parameters.Add("@paramDefect ", OleDbType.Integer).Value = Defect;
                cmd.ExecuteNonQuery();
                con.Close();
                assemblePIC.Image = Properties.Resources.ok;
                count += 1;
                lblQty.Text = count.ToString();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.ToString());
                assemblePIC.Image = Properties.Resources.error;
                KanbanTimer.Stop();
                startBtn.Enabled = true;
                stopBtn.Enabled = false;
            }
        }

        /// <summary>
        /// This method Checks the bin
        /// </summary>
        private void CheckBin()
        {
            try
            {
                binPIC.Image = Properties.Resources.waiting;
                OleDbConnection con = new OleDbConnection(StationConnectionString);
                con.Open();
                OleDbCommand cmd = new OleDbCommand(KanbanCommands.CheckBin(), con);
                cmd.CommandType = CommandType.StoredProcedure;

                cmd.Parameters.Add("@paramStationId", OleDbType.Integer).Value = StationID;
                cmd.Parameters.Add("@paramDateProduction", OleDbType.Date).Value = DateTime.Now.ToString("yyyy-MM-dd");

                cmd.ExecuteNonQuery();
                con.Close();
                binPIC.Image = Properties.Resources.ok;
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.ToString());
                binPIC.Image = Properties.Resources.error;
                KanbanTimer.Stop();
                startBtn.Enabled = true;
                stopBtn.Enabled = false;
            }
        }

        /// <summary>
        /// This method starts a station
        /// </summary>
        private void StartStation()
        {
            ///// Getting the profile ID ///
            //try
            //{
            //    runningPIC.Image = Properties.Resources.waiting;
            //    OleDbConnection con = new OleDbConnection(StationConnectionString);
            //    con.Open();
            //    OleDbCommand cmd = new OleDbCommand(KanbanCommands.LoadProfileID(ProfileTimer), con);
            //    OleDbDataReader ConfigValue = null;
            //    ConfigValue = cmd.ExecuteReader();
            //    while (ConfigValue.Read())
            //    {
            //        ProfileID = Convert.ToInt32(ConfigValue["profile_id"]);
            //    }
            //    con.Close();
            //    runningPIC.Image = Properties.Resources.ok;
            //    startBtn.Enabled = true;
            //}
            //catch (Exception ex)
            //{
            //    MessageBox.Show(ex.ToString());
            //    runningPIC.Image = Properties.Resources.error;
            //}

            /// Starting station //
            try
            {
                runningPIC.Image = Properties.Resources.waiting;
                OleDbConnection con = new OleDbConnection(StationConnectionString);

                OleDbCommand cmd = new OleDbCommand(KanbanCommands.StartStation(), con);
                cmd.CommandType = CommandType.StoredProcedure;

                //Input
                OleDbParameter ProfileId = new OleDbParameter("@ParamProfileId", OleDbType.Integer);
                ProfileId.Value = ProfileID;
                ProfileId.Direction = ParameterDirection.Input;
                cmd.Parameters.Add(ProfileId);

                OleDbParameter ProductId = new OleDbParameter("@ParamProductId", OleDbType.Integer);
                ProductId.Value = ProductID;
                ProductId.Direction = ParameterDirection.Input;
                cmd.Parameters.Add(ProductId);

                ///Return Value
                OleDbParameter StationId = new OleDbParameter("@Result", OleDbType.Integer);
                StationId.Direction = ParameterDirection.ReturnValue;
                cmd.Parameters.Add(StationId);

                con.Open();

                ///Executing commands
                cmd.ExecuteNonQuery();

                con.Close();

                runningPIC.Image = Properties.Resources.ok;
                isRunning = true;

                StationID = Convert.ToInt32(StationId.Value);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.ToString());
                runningPIC.Image = Properties.Resources.error;
                isRunning = false;
            }
        }

        /// <summary>
        /// This method get connection string
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void KanbanStation_Load(object sender, EventArgs e)
        {
            StationConnectionString = KanbanConnection.KanbanConnection();
        }

        /// <summary>
        /// This method Load station values
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ConnectBtn_Click_1(object sender, EventArgs e)
        {
            if (!String.IsNullOrEmpty(StationConnectionString))
            {
                ProfileBox.Enabled = true;
                ProductBox.Enabled = true;
                loadProfile();
                loadproduct();
                ConnectBtn.Enabled = false;

                LoadProfileConfig();
                ProfileLoaded = true;
            }
        }



        /// <summary>
        /// Get the assembly time for the profile
        /// </summary>
        private void GetProfileTimer()
        {
            /// Getting the Profile Timer///
            try
            {
                runningPIC.Image = Properties.Resources.waiting;
                OleDbConnection con = new OleDbConnection(StationConnectionString);
                con.Open();
                OleDbCommand cmd = new OleDbCommand(KanbanCommands.LoadAssemblyTime(ProfileID), con);
                OleDbDataReader ConfigValue = null;
                ConfigValue = cmd.ExecuteReader();
                while (ConfigValue.Read())
                {
                    ProfileTimer = Convert.ToInt32(ConfigValue["avg_assembly_time"]);
                }
                con.Close();
                runningPIC.Image = Properties.Resources.ok;
                startBtn.Enabled = true;
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.ToString());
                runningPIC.Image = Properties.Resources.error;
            }
        }

            /// <summary>
            /// This method define if the lamp is defect or not
            /// </summary>
            /// <param name="sender"></param>
            /// <param name="e"></param>
        private int GetDefect()
        {
            try
            {
                double avgDefect = 0.00;
                int modNum;
                Random r = new Random();
                int rndNum;
                int result = 0;

                /// Getting the avg defect///
                runningPIC.Image = Properties.Resources.waiting;
                OleDbConnection con = new OleDbConnection(StationConnectionString);
                con.Open();
                OleDbCommand cmd = new OleDbCommand(KanbanCommands.LoadAvgDefect(ProfileID), con);
                OleDbDataReader ConfigValue = null;
                ConfigValue = cmd.ExecuteReader();
                while (ConfigValue.Read())
                {
                    avgDefect = Convert.ToDouble(ConfigValue["avg_defect"]);
                }
                con.Close();
                runningPIC.Image = Properties.Resources.ok;

                //Apply %
                rndNum = r.Next();
                modNum = (int)(1 / (avgDefect / 100));
                rndNum = rndNum % modNum;

                if (rndNum == 1)
                {
                    result = 0;
                }
                else
                {
                    result = 1;
                }
                return result;
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.ToString());
                runningPIC.Image = Properties.Resources.error;
                return 0;
            }

        }
    }
}