﻿/**
*@FILENAME		: StationRunner.cs
*@DESCRIPTION	: Kanban Runner application that simulates a kanban runner process.
*@VERSION		: April 20, 2017
*@PROGRAMMER	: Brian Hinds and Laura Fadrique
*/

using System;
using System.Data;
using System.Data.OleDb;
using System.Windows.Forms;

namespace KanbanStationRunner
{
    public partial class StationRunner : Form
    {
        /// <summary>
        /// Variables for the Station Runner Application
        /// </summary>
        private int RunnerTime = 0;

        private const int TimerToSeconds = 100;
        private const int TimerToMinutes = 1000;
        private int RunnerInterval = 0;
        private bool TimerScale = true;
        private string StationConnectionString = "";

        /// <summary>
        /// Instantiating the classes needed for the station runner
        /// </summary>
        private RunnerConnection KanbanRunnerConnection = new RunnerConnection();

        private RunnerCommands KanbanRunnerCommands = new RunnerCommands();

        public StationRunner()
        {
            InitializeComponent();
            startBtn.Enabled = false;
            stopBtn.Enabled = false;
        }

        /// <summary>
        /// This method loads the connection string
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void StationRunner_Load(object sender, EventArgs e)
        {
            StationConnectionString = KanbanRunnerConnection.Runner_Connection();
        }

        /// <summary>
        /// This method execute the connection method to load timer configuration
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void connectBTN_Click(object sender, EventArgs e)
        {
            if (!String.IsNullOrEmpty(StationConnectionString))
            {
                LoadRunnerTime();
                connectBTN.Enabled = false;
            }
        }

        /// <summary>
        /// This method retrieve the runner time and the timer scale
        /// </summary>
        private void LoadRunnerTime()
        {
            try
            {
                connectionPIC.Image = Properties.Resources.waiting;
                OleDbConnection con = new OleDbConnection(StationConnectionString);
                con.Open();
                OleDbCommand cmd = new OleDbCommand(KanbanRunnerCommands.LoadRunnerTime(), con);
                OleDbDataReader RunnerValue = null;
                RunnerValue = cmd.ExecuteReader();
                while (RunnerValue.Read())
                {
                    RunnerTime = Convert.ToInt32(RunnerValue["runner_time"]);
                    TimerScale = Convert.ToBoolean(RunnerValue["scale_flag"]);
                }
                con.Close();
                startBtn.Enabled = true;
                connectionPIC.Image = Properties.Resources.ok;
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.ToString());
                connectionPIC.Image = Properties.Resources.error;
            }
        }

        /// <summary>
        /// This method execute the runner procedure
        /// </summary>
        private void StartRunner()
        {
            try
            {
                refillingPic.Image = Properties.Resources.waiting;
                OleDbConnection con = new OleDbConnection(StationConnectionString);
                con.Open();
                OleDbCommand cmd = new OleDbCommand(KanbanRunnerCommands.StartRunner(), con);
                cmd.CommandType = CommandType.StoredProcedure;
                cmd.ExecuteNonQuery();
                con.Close();
                refillingPic.Image = Properties.Resources.ok;
            }
            catch (Exception ex)
            {
                RunnerTimer.Stop();
                MessageBox.Show(ex.ToString());
                refillingPic.Image = Properties.Resources.error;
                runningPic.Image = Properties.Resources.error;
            }
        }

        /// <summary>
        /// This method starts the runner timer
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void startBtn_Click(object sender, EventArgs e)
        {
            runningPic.Image = Properties.Resources.waiting;

            /// Not scale to real time ///
            if (TimerScale)
            {
                RunnerInterval = RunnerTime * TimerToSeconds;
            }

            /// Scale to real time ///
            if (!TimerScale)
            {
                RunnerInterval = RunnerTime * TimerToMinutes;
            }

            /// Assigning timer interval
            RunnerTimer.Interval = RunnerInterval;

            /// Starting timer ///
            RunnerTimer.Start();

            runningPic.Image = Properties.Resources.ok;

            startBtn.Enabled = false;

            stopBtn.Enabled = true;
        }

        /// <summary>
        /// This method stops the runner timer
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void stopBtn_Click(object sender, EventArgs e)
        {
            RunnerTimer.Stop();
            stopBtn.Enabled = false;
            startBtn.Enabled = true;
            runningPic.Image = Properties.Resources.standby;
            refillingPic.Image = Properties.Resources.standby;
        }

        /// <summary>
        /// This method execute the runner every interval
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void RunnerTimer_Tick(object sender, EventArgs e)
        {
            StartRunner();
        }
    }
}